const _ = require("lodash");
const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const ko = require("knockout");

const template = require("./content.pug");

const confirm = require("./confirm");
const MediaPackage = require("cattura-ui-library/src/lib/mediaPackage");
const {apiv1} = require("./api");

const MediaPackageEditor = require("cattura-media-package-editor");

let player;
let mediaPackage = ko.observable();
mediaPackage.subscribe(value => {
    if (!player)
    {
        return;
    }

    if (!value)
    {
        player.closeMediaPackage();
        return;
    }

    player.openMediaPackageForEditing(value);
});

let clips = ko.observable([]);
let isExportable = ko.computed(() => {
    // Must have clips
    const hasClips = clips && clips().length > 0;
    if (!hasClips) return false;
    
    // Must have mpkg
    const hasMpkg = mediaPackage && mediaPackage();
    if (!hasMpkg) return false;
    
    // Clip must not be the entire media package
    const has1Clip      = clips().length === 1;
    const clipIsAtStart = has1Clip && clips()[0].begin === 0;
    const clipIsAtEnd   = has1Clip && Math.abs(clips()[0].end - mediaPackage().metadata.durationMillis / 1000) < 1;
    if (clipIsAtStart && clipIsAtEnd) return false;
    
    return true;
});

function clear ()
{
    mediaPackage(null);
    clips([]);
}

function makeDialog (plugin, exportEditedMedia)
{
    return Dialog({
        open (sourceMediaPackage)
        {
            clear();
            this.show();
            if (!player)
            {
                player = new MediaPackageEditor({
                    element: this.getElement().find(".editor"),
                    confirmFunction: message => confirm.open(message)
                });
                
                player.addEventListener("clipsupdate", event => {
                    clips(event.clips);
                });
            }

            mediaPackage(sourceMediaPackage);
        },

        clips: clips,
        isExportable: isExportable,

        submit ()
        {
            if (!isExportable()) return;
            
            player.pause();
            
            exportEditedMedia.open({
                mediaPackage: mediaPackage(),
                clips: clips(),
                isKeepHighlights: true,
            }).then(result => {
                this.hide();
                this.dispatchEvent({type: "editorexport", result: result});
            });
        },
        
        close ()
        {
            this.hide();
            clear();
        }
    }, {
        template: template
    });
}

module.exports = makeDialog;