const moment = require("moment");
const _ = require("lodash");
const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const ko = require("knockout");

const template = require("./content.pug");

const {TimeRange, Timestamp} = require("cattura-ui-library/src/lib/processor");
const {apiv1} = require("./api");

let mediaPackage = ko.observable();
let clips = ko.observable();
let newTitle = ko.observable();
let isKeepHighlights = ko.observable(true);

const totalClipDuration = ko.computed(() => {
    return _(clips()).reduce((total, clip) => {
        return total.add(clip.end - clip.begin, "seconds");
    }, moment.duration());
});

const totalDurationRemoved = ko.computed(() => {
    if (!mediaPackage())
    {
        return moment.duration();
    }

    return mediaPackage().duration.clone().subtract(totalClipDuration());
});

let resolver = null;

function clear ()
{
    mediaPackage(null);
    clips([]);
    newTitle("");
}

function makeDialog (plugin)
{
    return Dialog({
        open (clipperResult)
        {
            mediaPackage(clipperResult.mediaPackage);
            clips(clipperResult.clips);
            isKeepHighlights(_.isUndefined(clipperResult.isKeepHighlights) ? true : clipperResult.isKeepHighlights);
            
            this.show();
            return new Promise((resolve, reject) => resolver = resolve);
        },

        mediaPackage: mediaPackage,
        clips: clips,
        totalClipDuration: totalClipDuration,
        totalDurationRemoved: totalDurationRemoved,
        newTitle: newTitle,

        submit ()
        {
            plugin.services.clipMediaPackage({
                mediaPackageId: mediaPackage().id,
                newTitle: newTitle(),
                isKeepHighlights: isKeepHighlights(),
                clips: _(clips()).map(clip => {
                    return new TimeRange({
                        start: new Timestamp({
                            timestamp: clip.begin * 1000,
                            timeunit: "MILLISECONDS"
                        }),
                        end: new Timestamp({
                            timestamp: clip.end * 1000,
                            timeunit: "MILLISECONDS"
                        })
                    });
                }).value()
            }).then(result => {
                resolver(result);
                
                this.hide();
                clear();
            });
        },
        
        close ()
        {
            this.hide();
            clear();
        }
    }, {
        template: template
    });
}

module.exports = makeDialog;
